package classinfo;

import java.io.*;
import java.util.Vector;

public
class ConstantInPool {
  public static final int CONSTANT_Null =	0;
  public static final int CONSTANT_Utf8 =	1;
  public static final int CONSTANT_Unicode =	2;
  public static final int CONSTANT_Integer =	3;
  public static final int CONSTANT_Float =	4;
  public static final int CONSTANT_Long =	5;
  public static final int CONSTANT_Double =	6;
  public static final int CONSTANT_Class =	7;
  public static final int CONSTANT_String =	8;
  public static final int CONSTANT_Fieldref =	9;
  public static final int CONSTANT_Methodref =	10;
  public static final int CONSTANT_InterfaceMethodref =	11;
  public static final int CONSTANT_NameAndType =	12;


  // attributes
  int constantType;
  Object[] constantValue = new Object[3];

  /**
   * make a constant whose type is CONSTANT_Utf8.
   */
  public ConstantInPool(String str) {
    this(CONSTANT_Utf8, str);
  }

  public ConstantInPool() {
    this.constantType = CONSTANT_Null;
  }

  public ConstantInPool(int tag) {
    this.constantType = tag;
  }

  public ConstantInPool(int tag, Object value) {
    this.constantType = tag;
    this.constantValue[0] = value;
  }

  public ConstantInPool(BufManager bufm, Vector CPTable)
	throws IOException, ClassFormatError {
    int length;

    bufm.clearBuffer();
    bufm.advance(1);
    constantType = bufm.parseInt(0, 1);

    bufm.clearBuffer();

    // parse each constant in pool
    switch (constantType) {
    case CONSTANT_Utf8:
      bufm.advance(2);
      length = bufm.parseInt(0, 2);
      bufm.clearBuffer();

      bufm.advance(length);
      constantValue[0] = bufm.parseString(0, length);

      break;
    case CONSTANT_Unicode:
      bufm.advance(2);
      length = bufm.parseInt(0, 2);
      bufm.clearBuffer();

      bufm.advance(length);
      length /= 2;	// correct?
      char[] charBuffer = new char[length];
      for (int i = 0; i < length; i++) {
	charBuffer[i] = bufm.parseChar(i << 1);
      }

      constantValue[0] = new String(charBuffer, 0, length);

      break;
    case CONSTANT_Integer:
      bufm.advance(4);
      constantValue[0] = new Integer(bufm.parseInt(0, 4));

      break;
    case CONSTANT_Float:
      bufm.advance(4);
      constantValue[0] = new Float(bufm.parseFloat(0, 4));

      break;
    case CONSTANT_Long:
      bufm.advance(8);
      constantValue[0] = new Long(bufm.parseLong(0, 8));

      break;
    case CONSTANT_Double:
      bufm.advance(8);
      constantValue[0] = new Double(bufm.parseDouble(0, 8));

      break;
    case CONSTANT_Class:
      bufm.advance(2);
      constantValue[0] =
	new ConstantPoolReference(bufm.parseInt(0, 2), CPTable);

      break;
    case CONSTANT_String:
      bufm.advance(2);
      constantValue[0] =
	new ConstantPoolReference(bufm.parseInt(0, 2), CPTable);

      break;
    case CONSTANT_Fieldref:
      bufm.advance(4);
      constantValue[0] =
	new ConstantPoolReference(bufm.parseInt(0, 2), CPTable);
      constantValue[1] = 
	new ConstantPoolReference(bufm.parseInt(2, 2), CPTable);

      break;
    case CONSTANT_Methodref:
      bufm.advance(4);
      constantValue[0] =
	new ConstantPoolReference(bufm.parseInt(0, 2), CPTable);
      constantValue[1] =
	new ConstantPoolReference(bufm.parseInt(2, 2), CPTable);

      break;
    case CONSTANT_InterfaceMethodref:
      bufm.advance(4);
      constantValue[0] =
	new ConstantPoolReference(bufm.parseInt(0, 2), CPTable);
      constantValue[1] =
	new ConstantPoolReference(bufm.parseInt(2, 2), CPTable);

      break;
    case CONSTANT_NameAndType:
      bufm.advance(4);
      constantValue[0] =
	new ConstantPoolReference(bufm.parseInt(0, 2), CPTable);
      constantValue[1] =
	new ConstantPoolReference(bufm.parseInt(2, 2), CPTable);

      break;
    default:
      throw new ClassFormatError("Illegal constant type: " + constantType);
    }
  }


  public void write(DataOutputStream out)
	throws java.io.IOException {
    out.writeByte((byte)this.constantType);

    switch (this.constantType) {
    case CONSTANT_Utf8:
      Utf utfForm = new Utf((String)constantValue[0]);
      byte[] utfBytes = new byte[utfForm.utfLength()];
      utfForm.getBytes(utfBytes);

      out.writeShort((short)utfForm.utfLength());
      out.write(utfBytes, 0, utfBytes.length);
      break;
    case CONSTANT_Unicode:
      String unicodeValue = (String)constantValue[0];

      out.writeShort((short)unicodeValue.length());
      out.writeChars(unicodeValue);
      break;
    case CONSTANT_Integer:
      out.writeInt(((Integer)constantValue[0]).intValue());
      break;
    case CONSTANT_Float:
      out.writeFloat(((Float)constantValue[0]).floatValue());
      break;
    case CONSTANT_Long:
      out.writeLong(((Long)constantValue[0]).longValue());
      break;
    case CONSTANT_Double:
      out.writeDouble(((Double)constantValue[0]).doubleValue());
      break;
    case CONSTANT_Class:
      out.writeShort((short)((ConstantPoolReference)constantValue[0]).index());
      break;
    case CONSTANT_String:
      out.writeShort((short)((ConstantPoolReference)constantValue[0]).index());
      break;
    case CONSTANT_Fieldref:
      out.writeShort((short)((ConstantPoolReference)constantValue[0]).index());
      out.writeShort((short)((ConstantPoolReference)constantValue[1]).index());
      break;
    case CONSTANT_Methodref:
      out.writeShort((short)((ConstantPoolReference)constantValue[0]).index());
      out.writeShort((short)((ConstantPoolReference)constantValue[1]).index());
      break;
    case CONSTANT_InterfaceMethodref:
      out.writeShort((short)((ConstantPoolReference)constantValue[0]).index());
      out.writeShort((short)((ConstantPoolReference)constantValue[1]).index());
      break;
    case CONSTANT_NameAndType:
      out.writeShort((short)((ConstantPoolReference)constantValue[0]).index());
      out.writeShort((short)((ConstantPoolReference)constantValue[1]).index());
      break;
    default:
      // not reached.
      throw new ClassFormatError("Illegal constant type: " + constantType);
    }
  }


  public int type() { return constantType; }

  public boolean is64bit() {
    if ((constantType == CONSTANT_Long) ||
	(constantType == CONSTANT_Double))
      return true;
    else
      return false;
  }

  public Object value() { return constantValue[0]; }
  public Object value(int index) { return constantValue[index]; }

  public String toString() { return toString(""); }

  public String toString(String indent) {
    StringBuffer buf = new StringBuffer(BufManager.STRING_BUFFER_SIZE);

    buf.append(indent + "Constant: " + typeString(this.type()) + "\n");
    int i = 0;
    while (this.value(i) != null) {
      buf.append(indent + "\t" + this.value(i).toString() + "\n");
      i++;
    }

    return buf.toString();
  }

  private static String typeString(int type) {
    String ret = null;

    switch (type) {
    case CONSTANT_Null:
      ret = "(null)";  break;
    case CONSTANT_Utf8:
      ret = "CONSTANT_Utf8";  break;
    case CONSTANT_Unicode:
      ret = "CONSTANT_Unicode";  break;
    case CONSTANT_Integer:
      ret = "CONSTANT_Integer";  break;
    case CONSTANT_Float:
      ret = "CONSTANT_Float";  break;
    case CONSTANT_Long:
      ret = "CONSTANT_Long";  break;
    case CONSTANT_Double:
      ret = "CONSTANT_Double";  break;
    case CONSTANT_Class:
      ret = "CONSTANT_Class";  break;
    case CONSTANT_String:
      ret = "CONSTANT_String";  break;
    case CONSTANT_Fieldref:
      ret = "CONSTANT_Fieldref";  break;
    case CONSTANT_Methodref:
      ret = "CONSTANT_Methodref";  break;
    case CONSTANT_InterfaceMethodref:
      ret = "CONSTANT_InterfaceMethodref";  break;
    case CONSTANT_NameAndType:
      ret = "CONSTANT_NameAndType";  break;
    default:
      ret = "UNKNOWN";  break;
    }

    return ret;
  }


  public ConstantPoolReference[] constantPoolRefs() {
    int nref = 0;
    ConstantPoolReference[] ret;

    switch (this.constantType) {
    case CONSTANT_Utf8:
    case CONSTANT_Unicode:
    case CONSTANT_Integer:
    case CONSTANT_Float:
    case CONSTANT_Long:
    case CONSTANT_Double:
      nref = 0;
      break;
    case CONSTANT_Class:
    case CONSTANT_String:
      nref = 1;
      break;
    case CONSTANT_Fieldref:
    case CONSTANT_Methodref:
    case CONSTANT_InterfaceMethodref:
    case CONSTANT_NameAndType:
      nref = 2;
      break;
    default:
      // not reached.
      throw new ClassFormatError("Illegal constant type: " + constantType);
    }

    ret = new ConstantPoolReference[nref];
    for (int i = 0; i < nref; i++) {
      ret[i] = (ConstantPoolReference)constantValue[i];
    }

    return ret;
  }
}
