package classinfo;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;

public
class Member {
  private int accessFlag;	// 2 bytes
  private ConstantPoolReference nameIndex;	// 2 bytes
  private ConstantPoolReference signatureIndex;	// 2 bytes
  private int attributesCount;	// 2 bytes

  private Attribute[] attributes;


  private String memberType;

  public Member(String memberType, BufManager bufm, Vector CPTable)
	throws IOException, ClassFormatError {
    this.memberType = memberType;

    int attributeName, attributeLength;	// per attribute

    bufm.clearBuffer();
    bufm.advance(6);

    accessFlag = bufm.parseInt(0, 2);
    nameIndex = new ConstantPoolReference(bufm.parseInt(2, 2), CPTable);
    signatureIndex = new ConstantPoolReference(bufm.parseInt(4, 2), CPTable);

    // attributes count
    bufm.clearBuffer();
    bufm.advance(2);
    attributesCount = bufm.parseInt(0, 2);
    attributes = new Attribute[attributesCount];

    // attributes
    for (int i = 0; i < attributesCount; i++) {
      bufm.clearBuffer();
      bufm.advance(6);

      attributes[i] = new Attribute(bufm.parseInt(0, 2),
					bufm.parseInt(2, 4),
					bufm, CPTable);
    }
  }


  public void write(DataOutputStream out)
	throws java.io.IOException {
    out.writeShort((short)accessFlag);
    out.writeShort((short)nameIndex.index());
    out.writeShort((short)signatureIndex.index());

    out.writeShort((short)attributesCount);
    for (int i = 0; i < attributesCount; i++) {
      attributes[i].write(out);
    }
  }


  public String toString() { return toString(""); }

  public String toString(String indent) {
    StringBuffer buf = new StringBuffer(BufManager.STRING_BUFFER_SIZE);

    buf.append(indent + this.memberType + ":\n");
    buf.append(indent +
	"\tAccess flag: 0x" + BufManager.intToHexString(accessFlag,4) +
	" (" + AccessFlags.toString((short)accessFlag) + ")\n");
    buf.append(indent + "\tName: " + nameIndex.toString() + "\n");
    buf.append(indent + "\tSignature: " + signatureIndex.toString() + "\n");
    buf.append(indent + "\tAttributes count: " + attributesCount + "\n");

    // attributes
    for (int i = 0; i < attributesCount; i++) {
      buf.append(attributes[i].toString(indent + "\t"));
    }

    return buf.toString();
  }
}
