package classinfo.attribute;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;
import classinfo.*;

public
class Code extends AttributeInfo {
  public static final String ATTRIBUTE_NAME = "Code";

  int maxStack;		// 2 bytes
  int maxLocals;	// 2 bytes
  int codeLength;	// 4 bytes
  byte code[];
  int exceptionTableLength;	// 2 bytes
  ExceptionTable[] exceptionTable;
  int attributesCount;	// 2 bytes
  Attribute[] attributes;


  public Code(ConstantPoolReference nameIndex, int length,
			BufManager bufm, Vector CPTable)
	throws IOException {
    super(nameIndex, length);

    // max stack, max locals, code length
    bufm.clearBuffer();
    bufm.advance(8);
    maxStack = bufm.parseInt(0, 2);
    maxLocals = bufm.parseInt(2, 2);
    codeLength = bufm.parseInt(4, 4);
	// ATTENTION: assume that code length is less than 2^31 -1

    // code
    code = new byte[(int)codeLength];
    bufm.advance(code, 0, (int)codeLength);

    // exception table length
    bufm.clearBuffer();
    bufm.advance(2);
    exceptionTableLength = bufm.parseInt(0, 2);

    // exception table
    exceptionTable = new ExceptionTable[exceptionTableLength];
    for (int i = 0; i < exceptionTableLength; i++) {
      exceptionTable[i] = new ExceptionTable(bufm, CPTable);
    }

    // attributes count
    bufm.clearBuffer();
    bufm.advance(2);
    attributesCount = bufm.parseInt(0, 2);
    attributes = new Attribute[attributesCount];

    // attributes
    for (int i = 0; i < attributesCount; i++) {
      bufm.clearBuffer();
      bufm.advance(6);

      attributes[i] = new Attribute(bufm.parseInt(0, 2),
					bufm.parseInt(2, 4),
					bufm, CPTable);
    }
  }


  public void write(DataOutputStream out)
	throws java.io.IOException {
    out.writeShort((short)maxStack);
    out.writeShort((short)maxLocals);
    out.writeInt((int)codeLength);

    out.write(code, 0, (int)codeLength);
	// ATTENTION: assume that code length is less than 2^31 -1

    out.writeShort((short)exceptionTableLength);
    for (int i = 0; i < exceptionTableLength; i++) {
      exceptionTable[i].write(out);
    }

    out.writeShort((short)attributesCount);
    for (int i = 0; i < attributesCount; i++) {
      attributes[i].write(out);
    }
  }

  public String toString() { return toString(""); }

  public String toString(String indent) {
    StringBuffer buf = new StringBuffer(AttributeInfo.STRING_BUFFER_SIZE);
    int i;

    buf.append(super.toString(indent));

    buf.append(indent + "\tMax stack: " + maxStack + "\n");
    buf.append(indent + "\tMax locals: " + maxLocals + "\n");
    buf.append(indent + "\tCode length: " + codeLength + "\n");
    buf.append(indent + "\tException table length: "
			+ exceptionTableLength + "\n");

    // ɤ 16 ʤǽ
    buf.append(indent + "\tCode:");
    for (i = 0; i < codeLength; i++) {
      if ((i % 16) == 0)
	buf.append("\n" + indent + "\t\t");
      else
	buf.append(" ");
      String hex = Integer.toHexString((int)code[i] & 0xff);
      if (hex.length() == 1)
	buf.append("0");
      buf.append(hex);
    }
    buf.append("\n");

    for (i = 0; i < exceptionTableLength; i++) {
      buf.append(exceptionTable[i].toString(indent + "\t"));
    }

    buf.append(indent + "\tAttributes count: " + attributesCount + "\n");

    for (i = 0; i < attributesCount; i++) {
      buf.append(attributes[i].toString(indent + "\t"));
    }

    return buf.toString();
  }
}
