package classinfo.attribute;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;
import classinfo.*;

public
class LocalVariableTable extends AttributeInfo {
  public static final String ATTRIBUTE_NAME = "LocalVariableTable";

  int tableLength;	// 2 bytes
  int[] startPC;	// each entry is 2 bytes
  int[] PClength;		// each entry is 2 bytes
  ConstantPoolReference[] name;		// each entry is 2 bytes
  ConstantPoolReference[] signature;	// each entry is 2 bytes
  int[] slot;		// each entry is 2 bytes


  public LocalVariableTable(ConstantPoolReference nameIndex, int length,
			BufManager bufm, Vector CPTable)
	throws IOException {
    super(nameIndex, length);

    bufm.clearBuffer();
    bufm.advance(2);

    // local variable table length
    tableLength = bufm.parseInt(0, 2);

    // local variable table
    startPC = new int[tableLength];
    PClength = new int[tableLength];
    name = new ConstantPoolReference[tableLength];
    signature = new ConstantPoolReference[tableLength];
    slot = new int[tableLength];

    for (int i = 0; i < tableLength; i++) {
      bufm.clearBuffer();
      bufm.advance(10);

      startPC[i] = bufm.parseInt(0, 2);
      PClength[i] = bufm.parseInt(2, 2);
      name[i] = new ConstantPoolReference(bufm.parseInt(4, 2), CPTable);
      signature[i] = new ConstantPoolReference(bufm.parseInt(6, 2), CPTable);
      slot[i] = bufm.parseInt(8, 2);
    }
  }


  public void write(DataOutputStream out)
	throws java.io.IOException {
    out.writeShort((short)tableLength);
    for (int i = 0; i < tableLength; i++) {
      out.writeShort((short)startPC[i]);
      out.writeShort((short)PClength[i]);
      out.writeShort((short)name[i].index());
      out.writeShort((short)signature[i].index());
      out.writeShort((short)slot[i]);
    }
  }


  public String toString() { return toString(""); }

  public String toString(String indent) {
    StringBuffer buf = new StringBuffer(AttributeInfo.STRING_BUFFER_SIZE);

    buf.append(super.toString(indent));

    indent = indent + "\t";
    buf.append(indent + "Local variable table length: " +
					tableLength + "\n");

    for (int i = 0; i < tableLength; i++) {
      buf.append(indent + "Local variable table:\n");

      buf.append(indent + "\tstart pc, length: " +
			startPC[i] + ", " + PClength[i] + "\n");
      buf.append(indent + "\tname: " + name[i].toString() + "\n");
      buf.append(indent + "\tsignature: " + signature[i].toString() + "\n");
      buf.append(indent + "\tslot: " + slot[i] + "\n");
    }

    return buf.toString();
  }
}
